from typing import Optional, Tuple

import extra_data

from cal_tools.enums import JungfrauGainMode, JungfrauSettings


def _old_settings_to_new(settings: str, index: int) -> str:
    """At the end of 2022, there is a sls update.
    - `settings.value` values was updated from 6 possible values to only 2.
    - `gainMode.value` dataset was created with 5 possible values.

    https://git.xfel.eu/karaboDevices/slsDetectors/-/commit/4433ae9c00edcca3309bec8b7515e0938f5f502c  # noqa
    
    This method is used to get the settings or mode that
    corresponds to old settings.value
    """
    settings_lookup = {
        # old setting:  new setting, new mode
        "dynamicgain": ("gain0", "dynamic"),
        "dynamichg0":  ("highgain0", "dynamic"),
        "fixgain1": ("gain0", "fixg1"),
        "fixgain2": ("gain0", "fixg2"),
        "forceswitchg1": ("gain0", "forceswitchg1"),
        "forceswitchg2": ("gain0", "forceswitchg2"),
    }
    return settings_lookup[settings][index]


class JungfrauCtrl():
    def __init__(
        self,
        run_dc: extra_data.DataCollection,
        ctrl_src: str,
    ):
        """Read slow data from RUN source.
        :param run_dir: EXtra-data RunDirectory DataCollection object.
        :param ctrl_src: Control source name for accessing run slow data.
        """
        self.run_dc = run_dc
        self.ctrl_src = ctrl_src
        # run settings and mode are read as raw settings
        self.run_settings = self._get_settings()
        self.run_mode = self._get_mode()

    def _get_settings(self) -> str:
        """Get JUNGFRAU run settings and mode."""
        try:
            return self.run_dc.get_run_value(self.ctrl_src, "settings")
        except extra_data.PropertyNameError:
            print("\'settings.value\' key "
                  "is not available for this run. "
                  "`run_settings` is set to `dynamicgain`")
            return "dynamicgain"  # old data are set to `dynamicgain`.

    def _get_mode(self) -> str:
        """Get run mode from `gainMode` dataset."""
        try:
            return self.run_dc.get_run_value(self.ctrl_src, "gainMode")
        except extra_data.PropertyNameError:
            return self.run_settings

    def get_memory_cells(self) -> Tuple[int, int]:
        n_storage_cells = int(self.run_dc.get_run_value(
            self.ctrl_src, "storageCells.value")) + 1
        storage_cell_st = int(self.run_dc.get_run_value(
            self.ctrl_src, "storageCellStart.value"))
        return n_storage_cells, storage_cell_st

    def get_bias_voltage(self) -> int:
        """Get Bias voltage value from RUN source."""
        if "highVoltage" in self.run_dc[self.ctrl_src]:
            return int(
                self.run_dc.get_run_value(self.ctrl_src, "highVoltage")[0])
        else:  # Old dataset "vHighVoltage" before end of 2022.
            return int(
                self.run_dc.get_run_value(self.ctrl_src, "vHighVoltage")[0])

    def get_integration_time(self) -> float:
        return(float(self.run_dc.get_run_value(
            self.ctrl_src, "exposureTime.value")) * 1e6)

    def get_gain_setting(self) -> int:
        """Get run gain settings to identify if run is in
        High CDS or Low CDS.
        - `1` if run_settings = highgain0.
        - `0` if run_settings = gain0 or None.
        """
        # Check if `run_settings` is of an old settings value
        # to convert into new settings value.
        if self.run_settings in [s.value for s in JungfrauSettings]:
            settings = self.run_settings
        else:
            settings = _old_settings_to_new(self.run_settings, 0)

        if settings == JungfrauSettings.HIGH_GAIN_0.value:
            return 1
        else:  # JungfrauSettings.GAIN_0
            return 0

    def get_gain_mode(self) -> int:
        """Get gain mode value. Fixed `1` or Adaptive `1`.
        - `0` if run_mode = dynamic, forceswitchg1, forceswitchg2, or None.
        - `1` if run_mode = fixg1 or fixg2.
        """
        # Check if run_mode is of an old settings to convert
        # into new mode value.
        if self.run_mode in [m.value for m in JungfrauGainMode]:
            mode = self.run_mode
        else:
            mode = _old_settings_to_new(self.run_mode, 1)
        
        if mode in [
            JungfrauGainMode.FIX_GAIN_1.value,
            JungfrauGainMode.FIX_GAIN_2.value,
        ]:
            return 1
        else:  # DYNAMIC, FORCE_SWITCH_G1, or FORCE_SWITCH_G2
            return 0
